#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains a class that represents a Telegram InputInvoiceMessageContent."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._inline.inputmessagecontent import InputMessageContent
from telegram._payment.labeledprice import LabeledPrice
from telegram._utils.argumentparsing import de_list_optional, parse_sequence_arg
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class InputInvoiceMessageContent(InputMessageContent):
    """
    Represents the content of a invoice message to be sent as the result of an inline query.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`title`, :attr:`description`, :attr:`payload`,
    :attr:`currency` and :attr:`prices` are equal.

    .. versionadded:: 13.5
    .. versionchanged:: 21.11
       :attr:`provider_token` is no longer considered for equality comparison.

    Args:
        title (:obj:`str`): Product name. :tg-const:`telegram.Invoice.MIN_TITLE_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_TITLE_LENGTH` characters.
        description (:obj:`str`): Product description.
            :tg-const:`telegram.Invoice.MIN_DESCRIPTION_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_DESCRIPTION_LENGTH` characters.
        payload (:obj:`str`): Bot-defined invoice payload.
            :tg-const:`telegram.Invoice.MIN_PAYLOAD_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_PAYLOAD_LENGTH` bytes. This will not be displayed
            to the user, use it for your internal processes.
        provider_token (:obj:`str`, optional): Payment provider token, obtained via
            `@Botfather <https://t.me/Botfather>`_. Pass an empty string for payments in
            |tg_stars|.

            .. versionchanged:: 21.11
                Bot API 7.4 made this parameter is optional and this is now reflected in the
                class signature.
        currency (:obj:`str`): Three-letter ISO 4217 currency code, see more on
            `currencies <https://core.telegram.org/bots/payments#supported-currencies>`_.
            Pass ``XTR`` for payments in |tg_stars|.
        prices (Sequence[:class:`telegram.LabeledPrice`]): Price breakdown, a list of
            components (e.g. product price, tax, discount, delivery cost, delivery tax, bonus,
            etc.). Must contain exactly one item for payments in |tg_stars|.

            .. versionchanged:: 20.0
                |sequenceclassargs|

        max_tip_amount (:obj:`int`, optional): The maximum accepted amount for tips in the
            *smallest units* of the currency (integer, **not** float/double). For example, for a
            maximum tip of ``US$ 1.45`` pass ``max_tip_amount = 145``. See the ``exp`` parameter in
            `currencies.json <https://core.telegram.org/bots/payments/currencies.json>`_, it
            shows the number of digits past the decimal point for each currency (2 for the majority
            of currencies). Defaults to ``0``. Not supported for payments in |tg_stars|.
        suggested_tip_amounts (Sequence[:obj:`int`], optional): An array of suggested
            amounts of tip in the *smallest units* of the currency (integer, **not** float/double).
            At most 4 suggested tip amounts can be specified. The suggested tip amounts must be
            positive, passed in a strictly increased order and must not exceed
            :attr:`max_tip_amount`.

            .. versionchanged:: 20.0

                * |tupleclassattrs|
                * |alwaystuple|

        provider_data (:obj:`str`, optional): An object for data about the invoice,
            which will be shared with the payment provider. A detailed description of the required
            fields should be provided by the payment provider.
        photo_url (:obj:`str`, optional): URL of the product photo for the invoice. Can be a photo
            of the goods or a marketing image for a service. People like it better when they see
            what they are paying for.
        photo_size (:obj:`int`, optional): Photo size.
        photo_width (:obj:`int`, optional): Photo width.
        photo_height (:obj:`int`, optional): Photo height.
        need_name (:obj:`bool`, optional): Pass :obj:`True`, if you require the user's full
            name to complete the order. Ignored for payments in |tg_stars|.
        need_phone_number (:obj:`bool`, optional): Pass :obj:`True`, if you require the user's
            phone number to complete the order. Ignored for payments in |tg_stars|.
        need_email (:obj:`bool`, optional): Pass :obj:`True`, if you require the user's email
            address to complete the order. Ignored for payments in |tg_stars|.
        need_shipping_address (:obj:`bool`, optional): Pass :obj:`True`, if you require the
            user's shipping address to complete the order. Ignored for payments in |tg_stars|
        send_phone_number_to_provider (:obj:`bool`, optional): Pass :obj:`True`, if user's
            phone number should be sent to provider. Ignored for payments in |tg_stars|.
        send_email_to_provider (:obj:`bool`, optional): Pass :obj:`True`, if user's email
            address should be sent to provider. Ignored for payments in |tg_stars|.
        is_flexible (:obj:`bool`, optional): Pass :obj:`True`, if the final price depends on
            the shipping method. Ignored for payments in |tg_stars|.

    Attributes:
        title (:obj:`str`): Product name. :tg-const:`telegram.Invoice.MIN_TITLE_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_TITLE_LENGTH` characters.
        description (:obj:`str`): Product description.
            :tg-const:`telegram.Invoice.MIN_DESCRIPTION_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_DESCRIPTION_LENGTH` characters.
        payload (:obj:`str`): Bot-defined invoice payload.
            :tg-const:`telegram.Invoice.MIN_PAYLOAD_LENGTH`-
            :tg-const:`telegram.Invoice.MAX_PAYLOAD_LENGTH` bytes. This will not be displayed
            to the user, use it for your internal processes.
        provider_token (:obj:`str`): Payment provider token, obtained via
            `@Botfather <https://t.me/Botfather>`_. Pass an empty string for payments in `Telegram
            Stars <https://t.me/BotNews/90>`_.
        currency (:obj:`str`): Three-letter ISO 4217 currency code, see more on
            `currencies <https://core.telegram.org/bots/payments#supported-currencies>`_.
            Pass ``XTR`` for payments in |tg_stars|.
        prices (tuple[:class:`telegram.LabeledPrice`]): Price breakdown, a list of
            components (e.g. product price, tax, discount, delivery cost, delivery tax, bonus,
            etc.). Must contain exactly one item for payments in |tg_stars|.

            .. versionchanged:: 20.0
                |tupleclassattrs|

        max_tip_amount (:obj:`int`): Optional. The maximum accepted amount for tips in the
            *smallest units* of the currency (integer, **not** float/double). For example, for a
            maximum tip of ``US$ 1.45`` ``max_tip_amount`` is ``145``. See the ``exp`` parameter in
            `currencies.json <https://core.telegram.org/bots/payments/currencies.json>`_, it
            shows the number of digits past the decimal point for each currency (2 for the majority
            of currencies). Defaults to ``0``. Not supported for payments in |tg_stars|.
        suggested_tip_amounts (tuple[:obj:`int`]): Optional. An array of suggested
            amounts of tip in the *smallest units* of the currency (integer, **not** float/double).
            At most 4 suggested tip amounts can be specified. The suggested tip amounts must be
            positive, passed in a strictly increased order and must not exceed
            :attr:`max_tip_amount`.

            .. versionchanged:: 20.0
                |tupleclassattrs|

        provider_data (:obj:`str`): Optional. An object for data about the invoice,
            which will be shared with the payment provider. A detailed description of the required
            fields should be provided by the payment provider.
        photo_url (:obj:`str`): Optional. URL of the product photo for the invoice. Can be a photo
            of the goods or a marketing image for a service. People like it better when they see
            what they are paying for.
        photo_size (:obj:`int`): Optional. Photo size.
        photo_width (:obj:`int`): Optional. Photo width.
        photo_height (:obj:`int`): Optional. Photo height.
        need_name (:obj:`bool`): Optional. Pass :obj:`True`, if you require the user's full name to
            complete the order. Ignored for payments in |tg_stars|.
        need_phone_number (:obj:`bool`): Optional. Pass :obj:`True`, if you require the user's
            phone number to complete the order. Ignored for payments in |tg_stars|.
        need_email (:obj:`bool`): Optional. Pass :obj:`True`, if you require the user's email
            address to complete the order. Ignored for payments in |tg_stars|.
        need_shipping_address (:obj:`bool`): Optional. Pass :obj:`True`, if you require the user's
            shipping address to complete the order. Ignored for payments in |tg_stars|.
        send_phone_number_to_provider (:obj:`bool`): Optional. Pass :obj:`True`, if user's phone
            number should be sent to provider. Ignored for payments in |tg_stars|.
        send_email_to_provider (:obj:`bool`): Optional. Pass :obj:`True`, if user's email address
            should be sent to provider. Ignored for payments in |tg_stars|.
        is_flexible (:obj:`bool`): Optional. Pass :obj:`True`, if the final price depends on the
            shipping method. Ignored for payments in |tg_stars|.

    """

    __slots__ = (
        "currency",
        "description",
        "is_flexible",
        "max_tip_amount",
        "need_email",
        "need_name",
        "need_phone_number",
        "need_shipping_address",
        "payload",
        "photo_height",
        "photo_size",
        "photo_url",
        "photo_width",
        "prices",
        "provider_data",
        "provider_token",
        "send_email_to_provider",
        "send_phone_number_to_provider",
        "suggested_tip_amounts",
        "title",
    )

    def __init__(
        self,
        title: str,
        description: str,
        payload: str,
        currency: str,
        prices: Sequence[LabeledPrice],
        provider_token: str | None = None,
        max_tip_amount: int | None = None,
        suggested_tip_amounts: Sequence[int] | None = None,
        provider_data: str | None = None,
        photo_url: str | None = None,
        photo_size: int | None = None,
        photo_width: int | None = None,
        photo_height: int | None = None,
        need_name: bool | None = None,
        need_phone_number: bool | None = None,
        need_email: bool | None = None,
        need_shipping_address: bool | None = None,
        send_phone_number_to_provider: bool | None = None,
        send_email_to_provider: bool | None = None,
        is_flexible: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        with self._unfrozen():
            # Required
            self.title: str = title
            self.description: str = description
            self.payload: str = payload
            self.currency: str = currency
            self.prices: tuple[LabeledPrice, ...] = parse_sequence_arg(prices)
            # Optionals
            self.provider_token: str | None = provider_token
            self.max_tip_amount: int | None = max_tip_amount
            self.suggested_tip_amounts: tuple[int, ...] = parse_sequence_arg(suggested_tip_amounts)
            self.provider_data: str | None = provider_data
            self.photo_url: str | None = photo_url
            self.photo_size: int | None = photo_size
            self.photo_width: int | None = photo_width
            self.photo_height: int | None = photo_height
            self.need_name: bool | None = need_name
            self.need_phone_number: bool | None = need_phone_number
            self.need_email: bool | None = need_email
            self.need_shipping_address: bool | None = need_shipping_address
            self.send_phone_number_to_provider: bool | None = send_phone_number_to_provider
            self.send_email_to_provider: bool | None = send_email_to_provider
            self.is_flexible: bool | None = is_flexible

            self._id_attrs = (
                self.title,
                self.description,
                self.payload,
                self.currency,
                self.prices,
            )

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "InputInvoiceMessageContent":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["prices"] = de_list_optional(data.get("prices"), LabeledPrice, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm
from zoneinfo import ZoneInfo

import pytest

from telegram import (
    BusinessConnection,
    BusinessIntro,
    BusinessLocation,
    BusinessMessagesDeleted,
    BusinessOpeningHours,
    BusinessOpeningHoursInterval,
    Chat,
    Location,
    Sticker,
    User,
)
from telegram._business import BusinessBotRights
from telegram._utils.datetime import UTC, to_timestamp
from tests.auxil.slots import mro_slots


class BusinessTestBase:
    id_ = "123"
    user = User(123, "test_user", False)
    user_chat_id = 123
    date = dtm.datetime.now(tz=UTC).replace(microsecond=0)
    can_change_gift_settings = True
    can_convert_gifts_to_stars = True
    can_delete_all_messages = True
    can_delete_sent_messages = True
    can_edit_bio = True
    can_edit_name = True
    can_edit_profile_photo = True
    can_edit_username = True
    can_manage_stories = True
    can_read_messages = True
    can_reply = True
    can_transfer_and_upgrade_gifts = True
    can_transfer_stars = True
    can_view_gifts_and_stars = True
    is_enabled = True
    message_ids = (123, 321)
    business_connection_id = "123"
    chat = Chat(123, "test_chat")
    title = "Business Title"
    message = "Business description"
    sticker = Sticker("sticker_id", "unique_id", 50, 50, True, False, Sticker.REGULAR)
    address = "address"
    location = Location(-23.691288, 46.788279)
    opening_minute = 0
    closing_minute = 60
    time_zone_name = "Country/City"
    opening_hours = [
        BusinessOpeningHoursInterval(opening, opening + 60) for opening in (0, 24 * 60)
    ]


@pytest.fixture(scope="module")
def business_bot_rights():
    return BusinessBotRights(
        can_change_gift_settings=BusinessTestBase.can_change_gift_settings,
        can_convert_gifts_to_stars=BusinessTestBase.can_convert_gifts_to_stars,
        can_delete_all_messages=BusinessTestBase.can_delete_all_messages,
        can_delete_sent_messages=BusinessTestBase.can_delete_sent_messages,
        can_edit_bio=BusinessTestBase.can_edit_bio,
        can_edit_name=BusinessTestBase.can_edit_name,
        can_edit_profile_photo=BusinessTestBase.can_edit_profile_photo,
        can_edit_username=BusinessTestBase.can_edit_username,
        can_manage_stories=BusinessTestBase.can_manage_stories,
        can_read_messages=BusinessTestBase.can_read_messages,
        can_reply=BusinessTestBase.can_reply,
        can_transfer_and_upgrade_gifts=BusinessTestBase.can_transfer_and_upgrade_gifts,
        can_transfer_stars=BusinessTestBase.can_transfer_stars,
        can_view_gifts_and_stars=BusinessTestBase.can_view_gifts_and_stars,
    )


@pytest.fixture(scope="module")
def business_connection(business_bot_rights):
    return BusinessConnection(
        BusinessTestBase.id_,
        BusinessTestBase.user,
        BusinessTestBase.user_chat_id,
        BusinessTestBase.date,
        BusinessTestBase.is_enabled,
        rights=business_bot_rights,
    )


@pytest.fixture(scope="module")
def business_messages_deleted():
    return BusinessMessagesDeleted(
        BusinessTestBase.business_connection_id,
        BusinessTestBase.chat,
        BusinessTestBase.message_ids,
    )


@pytest.fixture(scope="module")
def business_intro():
    return BusinessIntro(
        BusinessTestBase.title,
        BusinessTestBase.message,
        BusinessTestBase.sticker,
    )


@pytest.fixture(scope="module")
def business_location():
    return BusinessLocation(
        BusinessTestBase.address,
        BusinessTestBase.location,
    )


@pytest.fixture(scope="module")
def business_opening_hours_interval():
    return BusinessOpeningHoursInterval(
        BusinessTestBase.opening_minute,
        BusinessTestBase.closing_minute,
    )


@pytest.fixture(scope="module")
def business_opening_hours():
    return BusinessOpeningHours(
        BusinessTestBase.time_zone_name,
        BusinessTestBase.opening_hours,
    )


class TestBusinessBotRightsWithoutRequest(BusinessTestBase):
    def test_slot_behaviour(self, business_bot_rights):
        inst = business_bot_rights
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_to_dict(self, business_bot_rights):
        rights_dict = business_bot_rights.to_dict()

        assert isinstance(rights_dict, dict)
        assert rights_dict["can_reply"] is self.can_reply
        assert rights_dict["can_read_messages"] is self.can_read_messages
        assert rights_dict["can_delete_sent_messages"] is self.can_delete_sent_messages
        assert rights_dict["can_delete_all_messages"] is self.can_delete_all_messages
        assert rights_dict["can_edit_name"] is self.can_edit_name
        assert rights_dict["can_edit_bio"] is self.can_edit_bio
        assert rights_dict["can_edit_profile_photo"] is self.can_edit_profile_photo
        assert rights_dict["can_edit_username"] is self.can_edit_username
        assert rights_dict["can_change_gift_settings"] is self.can_change_gift_settings
        assert rights_dict["can_view_gifts_and_stars"] is self.can_view_gifts_and_stars
        assert rights_dict["can_convert_gifts_to_stars"] is self.can_convert_gifts_to_stars
        assert rights_dict["can_transfer_and_upgrade_gifts"] is self.can_transfer_and_upgrade_gifts
        assert rights_dict["can_transfer_stars"] is self.can_transfer_stars
        assert rights_dict["can_manage_stories"] is self.can_manage_stories

    def test_de_json(self):
        json_dict = {
            "can_reply": self.can_reply,
            "can_read_messages": self.can_read_messages,
            "can_delete_sent_messages": self.can_delete_sent_messages,
            "can_delete_all_messages": self.can_delete_all_messages,
            "can_edit_name": self.can_edit_name,
            "can_edit_bio": self.can_edit_bio,
            "can_edit_profile_photo": self.can_edit_profile_photo,
            "can_edit_username": self.can_edit_username,
            "can_change_gift_settings": self.can_change_gift_settings,
            "can_view_gifts_and_stars": self.can_view_gifts_and_stars,
            "can_convert_gifts_to_stars": self.can_convert_gifts_to_stars,
            "can_transfer_and_upgrade_gifts": self.can_transfer_and_upgrade_gifts,
            "can_transfer_stars": self.can_transfer_stars,
            "can_manage_stories": self.can_manage_stories,
        }

        rights = BusinessBotRights.de_json(json_dict, None)
        assert rights.can_reply is self.can_reply
        assert rights.can_read_messages is self.can_read_messages
        assert rights.can_delete_sent_messages is self.can_delete_sent_messages
        assert rights.can_delete_all_messages is self.can_delete_all_messages
        assert rights.can_edit_name is self.can_edit_name
        assert rights.can_edit_bio is self.can_edit_bio
        assert rights.can_edit_profile_photo is self.can_edit_profile_photo
        assert rights.can_edit_username is self.can_edit_username
        assert rights.can_change_gift_settings is self.can_change_gift_settings
        assert rights.can_view_gifts_and_stars is self.can_view_gifts_and_stars
        assert rights.can_convert_gifts_to_stars is self.can_convert_gifts_to_stars
        assert rights.can_transfer_and_upgrade_gifts is self.can_transfer_and_upgrade_gifts
        assert rights.can_transfer_stars is self.can_transfer_stars
        assert rights.can_manage_stories is self.can_manage_stories
        assert rights.api_kwargs == {}
        assert isinstance(rights, BusinessBotRights)

    def test_equality(self):
        rights1 = BusinessBotRights(
            can_reply=self.can_reply,
        )

        rights2 = BusinessBotRights(
            can_reply=True,
        )

        rights3 = BusinessBotRights(
            can_reply=True,
            can_read_messages=self.can_read_messages,
        )

        assert rights1 == rights2
        assert hash(rights1) == hash(rights2)
        assert rights1 is not rights2

        assert rights1 != rights3
        assert hash(rights1) != hash(rights3)


class TestBusinessConnectionWithoutRequest(BusinessTestBase):
    def test_slots(self, business_connection):
        bc = business_connection
        for attr in bc.__slots__:
            assert getattr(bc, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(bc)) == len(set(mro_slots(bc))), "duplicate slot"

    def test_de_json(self, business_bot_rights):
        json_dict = {
            "id": self.id_,
            "user": self.user.to_dict(),
            "user_chat_id": self.user_chat_id,
            "date": to_timestamp(self.date),
            "is_enabled": self.is_enabled,
            "rights": business_bot_rights.to_dict(),
        }
        bc = BusinessConnection.de_json(json_dict, None)
        assert bc.id == self.id_
        assert bc.user == self.user
        assert bc.user_chat_id == self.user_chat_id
        assert bc.date == self.date
        assert bc.is_enabled == self.is_enabled
        assert bc.rights == business_bot_rights
        assert bc.api_kwargs == {}
        assert isinstance(bc, BusinessConnection)

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot, business_bot_rights):
        json_dict = {
            "id": self.id_,
            "user": self.user.to_dict(),
            "user_chat_id": self.user_chat_id,
            "date": to_timestamp(self.date),
            "is_enabled": self.is_enabled,
            "rights": business_bot_rights.to_dict(),
        }
        chat_bot = BusinessConnection.de_json(json_dict, offline_bot)
        chat_bot_raw = BusinessConnection.de_json(json_dict, raw_bot)
        chat_bot_tz = BusinessConnection.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing tzinfo objects is not reliable
        date_offset = chat_bot_tz.date.utcoffset()
        date_offset_tz = tz_bot.defaults.tzinfo.utcoffset(chat_bot_tz.date.replace(tzinfo=None))

        assert chat_bot.date.tzinfo == UTC
        assert chat_bot_raw.date.tzinfo == UTC
        assert date_offset_tz == date_offset

    def test_to_dict(self, business_connection, business_bot_rights):
        bc_dict = business_connection.to_dict()
        assert isinstance(bc_dict, dict)
        assert bc_dict["id"] == self.id_
        assert bc_dict["user"] == self.user.to_dict()
        assert bc_dict["user_chat_id"] == self.user_chat_id
        assert bc_dict["date"] == to_timestamp(self.date)
        assert bc_dict["is_enabled"] == self.is_enabled
        assert bc_dict["rights"] == business_bot_rights.to_dict()

    def test_equality(self, business_bot_rights):
        bc1 = BusinessConnection(
            self.id_,
            self.user,
            self.user_chat_id,
            self.date,
            self.is_enabled,
            rights=business_bot_rights,
        )
        bc2 = BusinessConnection(
            self.id_,
            self.user,
            self.user_chat_id,
            self.date,
            self.is_enabled,
            rights=business_bot_rights,
        )
        bc3 = BusinessConnection(
            "321",
            self.user,
            self.user_chat_id,
            self.date,
            self.is_enabled,
            rights=business_bot_rights,
        )
        bc4 = BusinessConnection(
            self.id_,
            self.user,
            self.user_chat_id,
            self.date,
            self.is_enabled,
            rights=BusinessBotRights(),
        )

        assert bc1 == bc2
        assert hash(bc1) == hash(bc2)

        assert bc1 != bc3
        assert hash(bc1) != hash(bc3)

        assert bc1 != bc4
        assert hash(bc1) != hash(bc4)


class TestBusinessMessagesDeleted(BusinessTestBase):
    def test_slots(self, business_messages_deleted):
        bmd = business_messages_deleted
        for attr in bmd.__slots__:
            assert getattr(bmd, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(bmd)) == len(set(mro_slots(bmd))), "duplicate slot"

    def test_to_dict(self, business_messages_deleted):
        bmd_dict = business_messages_deleted.to_dict()
        assert isinstance(bmd_dict, dict)
        assert bmd_dict["message_ids"] == list(self.message_ids)
        assert bmd_dict["business_connection_id"] == self.business_connection_id
        assert bmd_dict["chat"] == self.chat.to_dict()

    def test_de_json(self):
        json_dict = {
            "business_connection_id": self.business_connection_id,
            "chat": self.chat.to_dict(),
            "message_ids": self.message_ids,
        }
        bmd = BusinessMessagesDeleted.de_json(json_dict, None)
        assert bmd.business_connection_id == self.business_connection_id
        assert bmd.chat == self.chat
        assert bmd.message_ids == self.message_ids
        assert bmd.api_kwargs == {}
        assert isinstance(bmd, BusinessMessagesDeleted)

    def test_equality(self):
        bmd1 = BusinessMessagesDeleted(self.business_connection_id, self.chat, self.message_ids)
        bmd2 = BusinessMessagesDeleted(self.business_connection_id, self.chat, self.message_ids)
        bmd3 = BusinessMessagesDeleted("1", Chat(4, "random"), [321, 123])

        assert bmd1 == bmd2
        assert hash(bmd1) == hash(bmd2)

        assert bmd1 != bmd3
        assert hash(bmd1) != hash(bmd3)


class TestBusinessIntroWithoutRequest(BusinessTestBase):
    def test_slot_behaviour(self, business_intro):
        intro = business_intro
        for attr in intro.__slots__:
            assert getattr(intro, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(intro)) == len(set(mro_slots(intro))), "duplicate slot"

    def test_to_dict(self, business_intro):
        intro_dict = business_intro.to_dict()
        assert isinstance(intro_dict, dict)
        assert intro_dict["title"] == self.title
        assert intro_dict["message"] == self.message
        assert intro_dict["sticker"] == self.sticker.to_dict()

    def test_de_json(self):
        json_dict = {
            "title": self.title,
            "message": self.message,
            "sticker": self.sticker.to_dict(),
        }
        intro = BusinessIntro.de_json(json_dict, None)
        assert intro.title == self.title
        assert intro.message == self.message
        assert intro.sticker == self.sticker
        assert intro.api_kwargs == {}
        assert isinstance(intro, BusinessIntro)

    def test_equality(self):
        intro1 = BusinessIntro(self.title, self.message, self.sticker)
        intro2 = BusinessIntro(self.title, self.message, self.sticker)
        intro3 = BusinessIntro("Other Business", self.message, self.sticker)

        assert intro1 == intro2
        assert hash(intro1) == hash(intro2)
        assert intro1 is not intro2

        assert intro1 != intro3
        assert hash(intro1) != hash(intro3)


class TestBusinessLocationWithoutRequest(BusinessTestBase):
    def test_slot_behaviour(self, business_location):
        inst = business_location
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_to_dict(self, business_location):
        blc_dict = business_location.to_dict()
        assert isinstance(blc_dict, dict)
        assert blc_dict["address"] == self.address
        assert blc_dict["location"] == self.location.to_dict()

    def test_de_json(self):
        json_dict = {
            "address": self.address,
            "location": self.location.to_dict(),
        }
        blc = BusinessLocation.de_json(json_dict, None)
        assert blc.address == self.address
        assert blc.location == self.location
        assert blc.api_kwargs == {}
        assert isinstance(blc, BusinessLocation)

    def test_equality(self):
        blc1 = BusinessLocation(self.address, self.location)
        blc2 = BusinessLocation(self.address, self.location)
        blc3 = BusinessLocation("Other Address", self.location)

        assert blc1 == blc2
        assert hash(blc1) == hash(blc2)
        assert blc1 is not blc2

        assert blc1 != blc3
        assert hash(blc1) != hash(blc3)


class TestBusinessOpeningHoursIntervalWithoutRequest(BusinessTestBase):
    def test_slot_behaviour(self, business_opening_hours_interval):
        inst = business_opening_hours_interval
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_to_dict(self, business_opening_hours_interval):
        bohi_dict = business_opening_hours_interval.to_dict()
        assert isinstance(bohi_dict, dict)
        assert bohi_dict["opening_minute"] == self.opening_minute
        assert bohi_dict["closing_minute"] == self.closing_minute

    def test_de_json(self):
        json_dict = {
            "opening_minute": self.opening_minute,
            "closing_minute": self.closing_minute,
        }
        bohi = BusinessOpeningHoursInterval.de_json(json_dict, None)
        assert bohi.opening_minute == self.opening_minute
        assert bohi.closing_minute == self.closing_minute
        assert bohi.api_kwargs == {}
        assert isinstance(bohi, BusinessOpeningHoursInterval)

    def test_equality(self):
        bohi1 = BusinessOpeningHoursInterval(self.opening_minute, self.closing_minute)
        bohi2 = BusinessOpeningHoursInterval(self.opening_minute, self.closing_minute)
        bohi3 = BusinessOpeningHoursInterval(61, 100)

        assert bohi1 == bohi2
        assert hash(bohi1) == hash(bohi2)
        assert bohi1 is not bohi2

        assert bohi1 != bohi3
        assert hash(bohi1) != hash(bohi3)

    @pytest.mark.parametrize(
        ("opening_minute", "expected"),
        [  # openings per docstring
            (8 * 60, (0, 8, 0)),
            (24 * 60, (1, 0, 0)),
            (6 * 24 * 60, (6, 0, 0)),
        ],
    )
    def test_opening_time(self, opening_minute, expected):
        bohi = BusinessOpeningHoursInterval(opening_minute, -0)

        opening_time = bohi.opening_time
        assert opening_time == expected

        cached = bohi.opening_time
        assert cached is opening_time

    @pytest.mark.parametrize(
        ("closing_minute", "expected"),
        [  # closings per docstring
            (20 * 60 + 30, (0, 20, 30)),
            (2 * 24 * 60 - 1, (1, 23, 59)),
            (7 * 24 * 60 - 2, (6, 23, 58)),
        ],
    )
    def test_closing_time(self, closing_minute, expected):
        bohi = BusinessOpeningHoursInterval(-0, closing_minute)

        closing_time = bohi.closing_time
        assert closing_time == expected

        cached = bohi.closing_time
        assert cached is closing_time


class TestBusinessOpeningHoursWithoutRequest(BusinessTestBase):
    def test_slot_behaviour(self, business_opening_hours):
        inst = business_opening_hours
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_to_dict(self, business_opening_hours):
        boh_dict = business_opening_hours.to_dict()
        assert isinstance(boh_dict, dict)
        assert boh_dict["time_zone_name"] == self.time_zone_name
        assert boh_dict["opening_hours"] == [opening.to_dict() for opening in self.opening_hours]

    def test_de_json(self):
        json_dict = {
            "time_zone_name": self.time_zone_name,
            "opening_hours": [opening.to_dict() for opening in self.opening_hours],
        }
        boh = BusinessOpeningHours.de_json(json_dict, None)
        assert boh.time_zone_name == self.time_zone_name
        assert boh.opening_hours == tuple(self.opening_hours)
        assert boh.api_kwargs == {}
        assert isinstance(boh, BusinessOpeningHours)

    def test_equality(self):
        boh1 = BusinessOpeningHours(self.time_zone_name, self.opening_hours)
        boh2 = BusinessOpeningHours(self.time_zone_name, self.opening_hours)
        boh3 = BusinessOpeningHours("Other/Timezone", self.opening_hours)

        assert boh1 == boh2
        assert hash(boh1) == hash(boh2)
        assert boh1 is not boh2

        assert boh1 != boh3
        assert hash(boh1) != hash(boh3)

    class TestBusinessOpeningHoursGetOpeningHoursForDayWithoutRequest:
        @pytest.fixture
        def sample_opening_hours(self):
            # Monday 8am-8:30pm (480-1230)
            # Tuesday 24 hours (1440-2879)
            # Sunday 12am-11:58pm (8640-10078)
            intervals = [
                BusinessOpeningHoursInterval(480, 1230),  # Monday 8am-8:30pm
                BusinessOpeningHoursInterval(1440, 2879),  # Tuesday 24 hours
                BusinessOpeningHoursInterval(8640, 10078),  # Sunday 12am-11:58pm
            ]
            return BusinessOpeningHours(time_zone_name="UTC", opening_hours=intervals)

        def test_monday_opening_hours(self, sample_opening_hours):
            # Test for Monday
            test_date = dtm.date(2023, 11, 6)  # Monday
            time_zone = ZoneInfo("UTC")
            result = sample_opening_hours.get_opening_hours_for_day(test_date, time_zone)

            expected = (
                (
                    dtm.datetime(2023, 11, 6, 8, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 6, 20, 30, tzinfo=time_zone),
                ),
            )

            assert result == expected

        def test_tuesday_24_hours(self, sample_opening_hours):
            # Test for Tuesday (24 hours)
            test_date = dtm.date(2023, 11, 7)  # Tuesday
            time_zone = ZoneInfo("UTC")
            result = sample_opening_hours.get_opening_hours_for_day(test_date, time_zone)

            expected = (
                (
                    dtm.datetime(2023, 11, 7, 0, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 7, 23, 59, tzinfo=time_zone),
                ),
            )

            assert result == expected

        def test_sunday_opening_hours(self, sample_opening_hours):
            # Test for Sunday
            test_date = dtm.date(2023, 11, 12)  # Sunday
            time_zone = ZoneInfo("UTC")
            result = sample_opening_hours.get_opening_hours_for_day(test_date, time_zone)

            expected = (
                (
                    dtm.datetime(2023, 11, 12, 0, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 12, 23, 58, tzinfo=time_zone),
                ),
            )

            assert result == expected

        def test_day_with_no_opening_hours(self, sample_opening_hours):
            # Test for Wednesday (no opening hours defined)
            test_date = dtm.date(2023, 11, 8)  # Wednesday
            time_zone = ZoneInfo("UTC")
            result = sample_opening_hours.get_opening_hours_for_day(test_date, time_zone)

            assert result == ()

        def test_multiple_intervals_same_day(self):
            # Test with multiple intervals on the same day
            intervals = [
                # unsorted on purpose to check that the sorting works (even though this is
                # currently undocumented behaviour)
                BusinessOpeningHoursInterval(900, 1230),  # Monday 3pm-8:30pm
                BusinessOpeningHoursInterval(480, 720),  # Monday 8am-12pm
            ]
            opening_hours = BusinessOpeningHours(time_zone_name="UTC", opening_hours=intervals)

            test_date = dtm.date(2023, 11, 6)  # Monday
            time_zone = ZoneInfo("UTC")
            result = opening_hours.get_opening_hours_for_day(test_date, time_zone)

            expected = (
                (
                    dtm.datetime(2023, 11, 6, 8, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 6, 12, 0, tzinfo=time_zone),
                ),
                (
                    dtm.datetime(2023, 11, 6, 15, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 6, 20, 30, tzinfo=time_zone),
                ),
            )

            assert result == expected

        @pytest.mark.parametrize("input_type", [str, ZoneInfo])
        def test_timezone_conversion(self, sample_opening_hours, input_type):
            # Test that timezone is properly applied
            test_date = dtm.date(2023, 11, 6)  # Monday
            time_zone = input_type("America/New_York")
            zone_info = ZoneInfo("America/New_York")
            result = sample_opening_hours.get_opening_hours_for_day(test_date, time_zone)

            expected = (
                (
                    dtm.datetime(2023, 11, 6, 3, 0, tzinfo=zone_info),
                    dtm.datetime(2023, 11, 6, 15, 30, tzinfo=zone_info),
                ),
            )

            assert result == expected
            assert result[0][0].tzinfo == zone_info
            assert result[0][1].tzinfo == zone_info

        def test_timezone_conversation_changing_date(self):
            # test for the edge case where the returned time is on a different date in the target
            # timezone than in the business timezone
            intervals = [
                BusinessOpeningHoursInterval(60, 120),  # Monday 1am-2am UTC
            ]
            opening_hours = BusinessOpeningHours(time_zone_name="UTC", opening_hours=intervals)
            test_date = dtm.date(2023, 11, 6)  # Monday
            time_zone = ZoneInfo("America/New_York")  # UTC-5, so 1am UTC is 8pm previous day
            result = opening_hours.get_opening_hours_for_day(test_date, time_zone)
            expected = (
                (
                    dtm.datetime(2023, 11, 5, 20, 0, tzinfo=time_zone),
                    dtm.datetime(2023, 11, 5, 21, 0, tzinfo=time_zone),
                ),
            )
            assert result == expected

        def test_no_timezone_provided(self, sample_opening_hours):
            # Test when no timezone is provided
            test_date = dtm.date(2023, 11, 6)  # Monday
            result = sample_opening_hours.get_opening_hours_for_day(test_date)

            expected = (
                (
                    dtm.datetime(
                        2023,
                        11,
                        6,
                        8,
                        0,
                        tzinfo=ZoneInfo(sample_opening_hours.time_zone_name),
                    ),
                    dtm.datetime(
                        2023,
                        11,
                        6,
                        20,
                        30,
                        tzinfo=ZoneInfo(sample_opening_hours.time_zone_name),
                    ),
                ),
            )

            assert result == expected

    class TestBusinessOpeningHoursIsOpenWithoutRequest:
        @pytest.fixture
        def sample_opening_hours(self):
            # Monday 8am-8:30pm (480-1230)
            # Tuesday 24 hours (1440-2879)
            # Sunday 12am-11:59pm (8640-10079)
            intervals = [
                BusinessOpeningHoursInterval(480, 1230),  # Monday 8am-8:30pm UTC
                BusinessOpeningHoursInterval(1440, 2879),  # Tuesday 24 hours UTC
                BusinessOpeningHoursInterval(8640, 10079),  # Sunday 12am-11:59pm UTC
            ]
            return BusinessOpeningHours(time_zone_name="UTC", opening_hours=intervals)

        def test_is_open_during_business_hours(self, sample_opening_hours):
            # Monday 10am UTC (within 8am-8:30pm)
            dt = dtm.datetime(2023, 11, 6, 10, 0, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is True

        def test_is_open_at_opening_time(self, sample_opening_hours):
            # Monday exactly 8am UTC
            dt = dtm.datetime(2023, 11, 6, 8, 0, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is True

        def test_is_closed_at_closing_time(self, sample_opening_hours):
            # Monday exactly 8:30pm UTC (closing time is exclusive)
            dt = dtm.datetime(2023, 11, 6, 20, 30, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is False

        def test_is_closed_outside_business_hours(self, sample_opening_hours):
            # Monday 7am UTC (before opening)
            dt = dtm.datetime(2023, 11, 6, 7, 0, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is False

        def test_is_open_24h_day(self, sample_opening_hours):
            # Tuesday 3am UTC (24h opening)
            dt = dtm.datetime(2023, 11, 7, 3, 0, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is True

        def test_is_closed_on_day_with_no_hours(self, sample_opening_hours):
            # Wednesday (no opening hours)
            dt = dtm.datetime(2023, 11, 8, 12, 0, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is False

        def test_timezone_conversion(self, sample_opening_hours):
            # Monday 5am EDT is 10am UTC (should be open)
            dt = dtm.datetime(2023, 11, 6, 5, 0, tzinfo=ZoneInfo("America/New_York"))
            assert sample_opening_hours.is_open(dt) is True

            # Monday 2am EDT is 7am UTC (should be closed)
            dt = dtm.datetime(2023, 11, 6, 2, 0, tzinfo=ZoneInfo("America/New_York"))
            assert sample_opening_hours.is_open(dt) is False

        def test_naive_datetime_uses_business_timezone(self, sample_opening_hours):
            # Naive datetime - should be interpreted as UTC (business timezone)
            dt = dtm.datetime(2023, 11, 6, 10, 0)  # 10am naive
            assert sample_opening_hours.is_open(dt) is True

        def test_boundary_conditions(self, sample_opening_hours):
            # Sunday 11:58pm UTC (should be open)
            dt = dtm.datetime(2023, 11, 12, 23, 58, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is True

            # Sunday 11:59pm UTC (should be closed)
            dt = dtm.datetime(2023, 11, 12, 23, 59, tzinfo=ZoneInfo("UTC"))
            assert sample_opening_hours.is_open(dt) is False



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Dice, PaidMessagePriceChanged
from tests.auxil.slots import mro_slots


class PaidMessagePriceChangedTestBase:
    paid_message_star_count = 291


@pytest.fixture(scope="module")
def paid_message_price_changed():
    return PaidMessagePriceChanged(PaidMessagePriceChangedTestBase.paid_message_star_count)


class TestPaidMessagePriceChangedWithoutRequest(PaidMessagePriceChangedTestBase):
    def test_slot_behaviour(self, paid_message_price_changed):
        for attr in paid_message_price_changed.__slots__:
            assert getattr(paid_message_price_changed, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(paid_message_price_changed)) == len(
            set(mro_slots(paid_message_price_changed))
        ), "duplicate slot"

    def test_to_dict(self, paid_message_price_changed):
        pmpc_dict = paid_message_price_changed.to_dict()
        assert isinstance(pmpc_dict, dict)
        assert pmpc_dict["paid_message_star_count"] == self.paid_message_star_count

    def test_de_json(self, offline_bot):
        json_dict = {"paid_message_star_count": self.paid_message_star_count}
        pmpc = PaidMessagePriceChanged.de_json(json_dict, offline_bot)
        assert isinstance(pmpc, PaidMessagePriceChanged)
        assert pmpc.paid_message_star_count == self.paid_message_star_count
        assert pmpc.api_kwargs == {}

    def test_equality(self):
        pmpc1 = PaidMessagePriceChanged(self.paid_message_star_count)
        pmpc2 = PaidMessagePriceChanged(self.paid_message_star_count)
        pmpc3 = PaidMessagePriceChanged(3)
        dice = Dice(5, "emoji")

        assert pmpc1 == pmpc2
        assert hash(pmpc1) == hash(pmpc2)

        assert pmpc1 != pmpc3
        assert hash(pmpc1) != hash(pmpc3)

        assert pmpc1 != dice
        assert hash(pmpc1) != hash(dice)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Helper utilities around Telegram Objects first_name, last_name and username.
.. versionadded:: 22.4

Warning:
    Contents of this module are intended to be used internally by the library and *not* by the
    user. Changes to this module are not considered breaking changes and may not be documented in
    the changelog.
"""

from typing import TYPE_CHECKING, Protocol, TypeVar, overload

TeleUserLike = TypeVar("TeleUserLike", bound="UserLike")
TeleUserLikeOptional = TypeVar("TeleUserLikeOptional", bound="UserLikeOptional")

if TYPE_CHECKING:
    from typing import type_check_only

    @type_check_only
    class UserLike(Protocol):
        first_name: str
        last_name: str | None
        username: str | None

    @type_check_only
    class UserLikeOptional(Protocol):
        first_name: str | None
        last_name: str | None
        username: str | None


@overload
def get_name(userlike: TeleUserLike) -> str: ...
@overload
def get_name(userlike: TeleUserLikeOptional) -> str | None: ...


def get_name(userlike: TeleUserLike | TeleUserLikeOptional) -> str | None:
    """Returns ``username`` prefixed with "@". If  ``username`` is not available, calls
    :func:`get_full_name` below`.
    """
    if userlike.username:
        return f"@{userlike.username}"
    return get_full_name(userlike=userlike)


@overload
def get_full_name(userlike: TeleUserLike) -> str: ...
@overload
def get_full_name(userlike: TeleUserLikeOptional) -> str | None: ...


def get_full_name(userlike: TeleUserLike | TeleUserLikeOptional) -> str | None:
    """
    If parameter ``first_name`` is not :obj:`None`, gives
    ``first_name`` followed by (if available) `UserLike.last_name`. Otherwise,
    :obj:`None` is returned.
    """
    if not userlike.first_name:
        return None
    if userlike.last_name:
        return f"{userlike.first_name} {userlike.last_name}"
    return userlike.first_name


# We isolate these TypeVars to accomodiate telegram objects with ``username``
# and no ``first_name`` or ``last_name`` (e.g ``ChatShared``)
TeleLinkable = TypeVar("TeleLinkable", bound="Linkable")
TeleLinkableOptional = TypeVar("TeleLinkableOptional", bound="LinkableOptional")

if TYPE_CHECKING:

    @type_check_only
    class Linkable(Protocol):
        username: str

    @type_check_only
    class LinkableOptional(Protocol):
        username: str | None


@overload
def get_link(linkable: TeleLinkable) -> str: ...
@overload
def get_link(linkable: TeleLinkableOptional) -> str | None: ...


def get_link(linkable: TeleLinkable | TeleLinkableOptional) -> str | None:
    """If ``username`` is available, returns a t.me link of the user/chat."""
    if linkable.username:
        return f"https://t.me/{linkable.username}"
    return None



#!/usr/bin/env python
# pylint: disable=unused-argument
# This program is dedicated to the public domain under the CC0 license.

"""
Simple Bot to print/download all incoming passport data

See https://telegram.org/blog/passport for info about what telegram passport is.

See https://github.com/python-telegram-bot/python-telegram-bot/wiki/Telegram-Passport
 for how to use Telegram Passport properly with python-telegram-bot.

Note:
To use Telegram Passport, you must install PTB via
`pip install "python-telegram-bot[passport]"`
"""

import logging
from pathlib import Path

from telegram import Update
from telegram.ext import Application, ContextTypes, MessageHandler, filters

# Enable logging

logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)

# set higher logging level for httpx to avoid all GET and POST requests being logged
logging.getLogger("httpx").setLevel(logging.WARNING)

logger = logging.getLogger(__name__)


async def msg(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Downloads and prints the received passport data."""
    # Retrieve passport data
    passport_data = update.message.passport_data
    # If our nonce doesn't match what we think, this Update did not originate from us
    # Ideally you would randomize the nonce on the server
    if passport_data.decrypted_credentials.nonce != "thisisatest":
        return

    # Print the decrypted credential data
    # For all elements
    # Print their decrypted data
    # Files will be downloaded to current directory
    for data in passport_data.decrypted_data:  # This is where the data gets decrypted
        if data.type == "phone_number":
            logger.info("Phone: %s", data.phone_number)
        elif data.type == "email":
            logger.info("Email: %s", data.email)
        if data.type in (
            "personal_details",
            "passport",
            "driver_license",
            "identity_card",
            "internal_passport",
            "address",
        ):
            logger.info(data.type, data.data)
        if data.type in (
            "utility_bill",
            "bank_statement",
            "rental_agreement",
            "passport_registration",
            "temporary_registration",
        ):
            logger.info(data.type, len(data.files), "files")
            for file in data.files:
                actual_file = await file.get_file()
                logger.info(actual_file)
                await actual_file.download_to_drive()
        if (
            data.type in ("passport", "driver_license", "identity_card", "internal_passport")
            and data.front_side
        ):
            front_file = await data.front_side.get_file()
            logger.info(data.type, front_file)
            await front_file.download_to_drive()
        if data.type in ("driver_license" and "identity_card") and data.reverse_side:
            reverse_file = await data.reverse_side.get_file()
            logger.info(data.type, reverse_file)
            await reverse_file.download_to_drive()
        if (
            data.type in ("passport", "driver_license", "identity_card", "internal_passport")
            and data.selfie
        ):
            selfie_file = await data.selfie.get_file()
            logger.info(data.type, selfie_file)
            await selfie_file.download_to_drive()
        if data.translation and data.type in (
            "passport",
            "driver_license",
            "identity_card",
            "internal_passport",
            "utility_bill",
            "bank_statement",
            "rental_agreement",
            "passport_registration",
            "temporary_registration",
        ):
            logger.info(data.type, len(data.translation), "translation")
            for file in data.translation:
                actual_file = await file.get_file()
                logger.info(actual_file)
                await actual_file.download_to_drive()


def main() -> None:
    """Start the bot."""
    # Create the Application and pass it your token and private key
    private_key = Path("private.key")
    application = (
        Application.builder().token("TOKEN").private_key(private_key.read_bytes()).build()
    )

    # On messages that include passport data call msg
    application.add_handler(MessageHandler(filters.PASSPORT_DATA, msg))

    # Run the bot until the user presses Ctrl-C
    application.run_polling(allowed_updates=Update.ALL_TYPES)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import (
    ForceReply,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
)
from telegram.constants import KeyboardButtonStyle
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_keyboard_markup():
    return InlineKeyboardMarkup(InlineKeyboardMarkupTestBase.inline_keyboard)


class InlineKeyboardMarkupTestBase:
    inline_keyboard = [
        [
            InlineKeyboardButton(text="button1", callback_data="data1"),
            InlineKeyboardButton(text="button2", callback_data="data2"),
        ]
    ]


class TestInlineKeyboardMarkupWithoutRequest(InlineKeyboardMarkupTestBase):
    def test_slot_behaviour(self, inline_keyboard_markup):
        inst = inline_keyboard_markup
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_to_dict(self, inline_keyboard_markup):
        inline_keyboard_markup_dict = inline_keyboard_markup.to_dict()

        assert isinstance(inline_keyboard_markup_dict, dict)
        assert inline_keyboard_markup_dict["inline_keyboard"] == [
            [self.inline_keyboard[0][0].to_dict(), self.inline_keyboard[0][1].to_dict()]
        ]

    def test_de_json(self):
        json_dict = {
            "inline_keyboard": [
                [
                    {"text": "start", "url": "http://google.com"},
                    {"text": "next", "callback_data": "abcd"},
                ],
                [{"text": "Cancel", "callback_data": "Cancel"}],
            ]
        }
        inline_keyboard_markup = InlineKeyboardMarkup.de_json(json_dict, None)
        assert inline_keyboard_markup.api_kwargs == {}

        assert isinstance(inline_keyboard_markup, InlineKeyboardMarkup)
        keyboard = inline_keyboard_markup.inline_keyboard
        assert len(keyboard) == 2
        assert len(keyboard[0]) == 2
        assert len(keyboard[1]) == 1

        assert isinstance(keyboard[0][0], InlineKeyboardButton)
        assert isinstance(keyboard[0][1], InlineKeyboardButton)
        assert isinstance(keyboard[1][0], InlineKeyboardButton)

        assert keyboard[0][0].text == "start"
        assert keyboard[0][0].url == "http://google.com"

    def test_equality(self):
        a = InlineKeyboardMarkup.from_column(
            [
                InlineKeyboardButton(label, callback_data="data")
                for label in ["button1", "button2", "button3"]
            ]
        )
        b = InlineKeyboardMarkup.from_column(
            [
                InlineKeyboardButton(label, callback_data="data")
                for label in ["button1", "button2", "button3"]
            ]
        )
        c = InlineKeyboardMarkup.from_column(
            [InlineKeyboardButton(label, callback_data="data") for label in ["button1", "button2"]]
        )
        d = InlineKeyboardMarkup.from_column(
            [
                InlineKeyboardButton(label, callback_data=label)
                for label in ["button1", "button2", "button3"]
            ]
        )
        e = InlineKeyboardMarkup.from_column(
            [InlineKeyboardButton(label, url=label) for label in ["button1", "button2", "button3"]]
        )
        f = InlineKeyboardMarkup(
            [
                [
                    InlineKeyboardButton(label, callback_data="data")
                    for label in ["button1", "button2"]
                ],
                [
                    InlineKeyboardButton(label, callback_data="data")
                    for label in ["button1", "button2"]
                ],
                [
                    InlineKeyboardButton(label, callback_data="data")
                    for label in ["button1", "button2"]
                ],
            ]
        )
        g = ReplyKeyboardMarkup.from_column(["button1", "button2", "button3"])

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)

        assert a != g
        assert hash(a) != hash(g)

    def test_from_button(self):
        inline_keyboard_markup = InlineKeyboardMarkup.from_button(
            InlineKeyboardButton(text="button1", callback_data="data1")
        ).inline_keyboard
        assert len(inline_keyboard_markup) == 1
        assert len(inline_keyboard_markup[0]) == 1

    def test_from_row(self):
        inline_keyboard_markup = InlineKeyboardMarkup.from_row(
            [
                InlineKeyboardButton(text="button1", callback_data="data1"),
                InlineKeyboardButton(text="button1", callback_data="data1"),
            ]
        ).inline_keyboard
        assert len(inline_keyboard_markup) == 1
        assert len(inline_keyboard_markup[0]) == 2

    def test_from_column(self):
        inline_keyboard_markup = InlineKeyboardMarkup.from_column(
            [
                InlineKeyboardButton(text="button1", callback_data="data1"),
                InlineKeyboardButton(text="button1", callback_data="data1"),
            ]
        ).inline_keyboard
        assert len(inline_keyboard_markup) == 2
        assert len(inline_keyboard_markup[0]) == 1
        assert len(inline_keyboard_markup[1]) == 1

    def test_expected_values(self, inline_keyboard_markup):
        assert inline_keyboard_markup.inline_keyboard == tuple(
            tuple(row) for row in self.inline_keyboard
        )

    def test_wrong_keyboard_inputs(self):
        with pytest.raises(ValueError, match="should be a sequence of sequences"):
            InlineKeyboardMarkup(
                [[InlineKeyboardButton("b1", "1")], InlineKeyboardButton("b2", "2")]
            )
        with pytest.raises(ValueError, match="should be a sequence of sequences"):
            InlineKeyboardMarkup("strings_are_not_allowed")
        with pytest.raises(ValueError, match="should be a sequence of sequences"):
            InlineKeyboardMarkup(["strings_are_not_allowed_in_the_rows_either"])
        with pytest.raises(ValueError, match="should be a sequence of sequences"):
            InlineKeyboardMarkup(InlineKeyboardButton("b1", "1"))
        with pytest.raises(ValueError, match="should be a sequence of sequences"):
            InlineKeyboardMarkup([[[InlineKeyboardButton("only_2d_array_is_allowed", "1")]]])

    async def test_expected_values_empty_switch(
        self, inline_keyboard_markup, offline_bot, monkeypatch
    ):
        async def make_assertion(
            url,
            data,
            reply_to_message_id=None,
            disable_notification=None,
            reply_markup=None,
            timeout=None,
            **kwargs,
        ):
            if reply_markup is not None:
                markups = (
                    InlineKeyboardMarkup,
                    ReplyKeyboardMarkup,
                    ForceReply,
                    ReplyKeyboardRemove,
                )
                if isinstance(reply_markup, markups):
                    data["reply_markup"] = reply_markup.to_dict()
                else:
                    data["reply_markup"] = reply_markup

            assert bool("'switch_inline_query': ''" in str(data["reply_markup"]))
            assert bool("'switch_inline_query_current_chat': ''" in str(data["reply_markup"]))

        inline_keyboard_markup.inline_keyboard[0][0]._unfreeze()
        inline_keyboard_markup.inline_keyboard[0][0].callback_data = None
        inline_keyboard_markup.inline_keyboard[0][0].switch_inline_query = ""
        inline_keyboard_markup.inline_keyboard[0][1]._unfreeze()
        inline_keyboard_markup.inline_keyboard[0][1].callback_data = None
        inline_keyboard_markup.inline_keyboard[0][1].switch_inline_query_current_chat = ""

        monkeypatch.setattr(offline_bot, "_send_message", make_assertion)
        await offline_bot.send_message(123, "test", reply_markup=inline_keyboard_markup)


class TestInlineKeyborardMarkupWithRequest(InlineKeyboardMarkupTestBase):
    async def test_send_message_with_inline_keyboard_markup(
        self, bot, chat_id, inline_keyboard_markup
    ):
        message = await bot.send_message(
            chat_id, "Testing InlineKeyboardMarkup", reply_markup=inline_keyboard_markup
        )

        assert message.text == "Testing InlineKeyboardMarkup"

    async def test_send_message_with_colored_inline_keyboard_button(self, bot, chat_id):
        markup = InlineKeyboardMarkup(
            [
                [
                    InlineKeyboardButton(
                        text="Colored Button",
                        callback_data="data1",
                        style=KeyboardButtonStyle.DANGER,
                    )
                ]
            ]
        )
        message = await bot.send_message(
            chat_id,
            "Testing colored InlineKeyboardButton",
            reply_markup=markup,
        )
        assert message.text == "Testing colored InlineKeyboardButton"
        button = message.reply_markup.inline_keyboard[0][0]
        assert button.style == "danger"



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    Chat,
    Checklist,
    ChecklistTask,
    ChecklistTasksAdded,
    ChecklistTasksDone,
    Dice,
    MessageEntity,
    User,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import ZERO_DATE
from tests.auxil.build_messages import make_message
from tests.auxil.slots import mro_slots


class ChecklistTaskTestBase:
    id = 42
    text = "here is a text"
    text_entities = [
        MessageEntity(type="bold", offset=0, length=4),
        MessageEntity(type="italic", offset=5, length=2),
    ]
    completed_by_user = User(id=1, first_name="Test", last_name="User", is_bot=False)
    completed_by_chat = Chat(id=-100, type=Chat.SUPERGROUP, title="Test Chat")
    completion_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


@pytest.fixture(scope="module")
def checklist_task():
    return ChecklistTask(
        id=ChecklistTaskTestBase.id,
        text=ChecklistTaskTestBase.text,
        text_entities=ChecklistTaskTestBase.text_entities,
        completed_by_user=ChecklistTaskTestBase.completed_by_user,
        completed_by_chat=ChecklistTaskTestBase.completed_by_chat,
        completion_date=ChecklistTaskTestBase.completion_date,
    )


class TestChecklistTaskWithoutRequest(ChecklistTaskTestBase):
    def test_slot_behaviour(self, checklist_task):
        for attr in checklist_task.__slots__:
            assert getattr(checklist_task, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(checklist_task)) == len(set(mro_slots(checklist_task))), (
            "duplicate slot"
        )

    def test_to_dict(self, checklist_task):
        clt_dict = checklist_task.to_dict()
        assert isinstance(clt_dict, dict)
        assert clt_dict["id"] == self.id
        assert clt_dict["text"] == self.text
        assert clt_dict["text_entities"] == [entity.to_dict() for entity in self.text_entities]
        assert clt_dict["completed_by_user"] == self.completed_by_user.to_dict()
        assert clt_dict["completed_by_chat"] == self.completed_by_chat.to_dict()
        assert clt_dict["completion_date"] == to_timestamp(self.completion_date)

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id,
            "text": self.text,
            "text_entities": [entity.to_dict() for entity in self.text_entities],
            "completed_by_user": self.completed_by_user.to_dict(),
            "completed_by_chat": self.completed_by_chat.to_dict(),
            "completion_date": to_timestamp(self.completion_date),
        }
        clt = ChecklistTask.de_json(json_dict, offline_bot)
        assert isinstance(clt, ChecklistTask)
        assert clt.id == self.id
        assert clt.text == self.text
        assert clt.text_entities == tuple(self.text_entities)
        assert clt.completed_by_user == self.completed_by_user
        assert clt.completed_by_chat == self.completed_by_chat
        assert clt.completion_date == self.completion_date
        assert clt.api_kwargs == {}

    def test_de_json_required_fields(self, offline_bot):
        json_dict = {
            "id": self.id,
            "text": self.text,
        }
        clt = ChecklistTask.de_json(json_dict, offline_bot)
        assert isinstance(clt, ChecklistTask)
        assert clt.id == self.id
        assert clt.text == self.text
        assert clt.text_entities == ()
        assert clt.completed_by_user is None
        assert clt.completion_date is None
        assert clt.api_kwargs == {}

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot):
        json_dict = {
            "id": self.id,
            "text": self.text,
            "completion_date": to_timestamp(self.completion_date),
        }
        clt_bot = ChecklistTask.de_json(json_dict, offline_bot)
        clt_bot_raw = ChecklistTask.de_json(json_dict, raw_bot)
        clt_bot_tz = ChecklistTask.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing tzinfo objects is not reliable
        completion_date_offset = clt_bot_tz.completion_date.utcoffset()
        completion_date_offset_tz = tz_bot.defaults.tzinfo.utcoffset(
            clt_bot_tz.completion_date.replace(tzinfo=None)
        )

        assert clt_bot.completion_date.tzinfo == UTC
        assert clt_bot_raw.completion_date.tzinfo == UTC
        assert completion_date_offset_tz == completion_date_offset

    @pytest.mark.parametrize(
        ("completion_date", "expected"),
        [
            (None, None),
            (0, ZERO_DATE),
            (1735689600, dtm.datetime(2025, 1, 1, tzinfo=UTC)),
        ],
    )
    def test_de_json_completion_date(self, offline_bot, completion_date, expected):
        json_dict = {
            "id": self.id,
            "text": self.text,
            "completion_date": completion_date,
        }
        clt = ChecklistTask.de_json(json_dict, offline_bot)
        assert isinstance(clt, ChecklistTask)
        assert clt.completion_date == expected

    def test_parse_entity(self, checklist_task):
        assert checklist_task.parse_entity(checklist_task.text_entities[0]) == "here"

    def test_parse_entities(self, checklist_task):
        assert checklist_task.parse_entities(MessageEntity.BOLD) == {
            checklist_task.text_entities[0]: "here"
        }
        assert checklist_task.parse_entities() == {
            checklist_task.text_entities[0]: "here",
            checklist_task.text_entities[1]: "is",
        }

    def test_equality(self, checklist_task):
        clt1 = checklist_task
        clt2 = ChecklistTask(
            id=self.id,
            text="other text",
        )
        clt3 = ChecklistTask(
            id=self.id + 1,
            text=self.text,
        )
        clt4 = Dice(value=1, emoji="🎲")

        assert clt1 == clt2
        assert hash(clt1) == hash(clt2)

        assert clt1 != clt3
        assert hash(clt1) != hash(clt3)

        assert clt1 != clt4
        assert hash(clt1) != hash(clt4)


class ChecklistTestBase:
    title = "Checklist Title"
    title_entities = [
        MessageEntity(type="bold", offset=0, length=9),
        MessageEntity(type="italic", offset=10, length=5),
    ]
    tasks = [
        ChecklistTask(
            id=1,
            text="Task 1",
        ),
        ChecklistTask(
            id=2,
            text="Task 2",
        ),
    ]
    others_can_add_tasks = True
    others_can_mark_tasks_as_done = False


@pytest.fixture(scope="module")
def checklist():
    return Checklist(
        title=ChecklistTestBase.title,
        title_entities=ChecklistTestBase.title_entities,
        tasks=ChecklistTestBase.tasks,
        others_can_add_tasks=ChecklistTestBase.others_can_add_tasks,
        others_can_mark_tasks_as_done=ChecklistTestBase.others_can_mark_tasks_as_done,
    )


class TestChecklistWithoutRequest(ChecklistTestBase):
    def test_slot_behaviour(self, checklist):
        for attr in checklist.__slots__:
            assert getattr(checklist, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(checklist)) == len(set(mro_slots(checklist))), "duplicate slot"

    def test_to_dict(self, checklist):
        cl_dict = checklist.to_dict()
        assert isinstance(cl_dict, dict)
        assert cl_dict["title"] == self.title
        assert cl_dict["title_entities"] == [entity.to_dict() for entity in self.title_entities]
        assert cl_dict["tasks"] == [task.to_dict() for task in self.tasks]
        assert cl_dict["others_can_add_tasks"] is self.others_can_add_tasks
        assert cl_dict["others_can_mark_tasks_as_done"] is self.others_can_mark_tasks_as_done

    def test_de_json(self, offline_bot):
        json_dict = {
            "title": self.title,
            "title_entities": [entity.to_dict() for entity in self.title_entities],
            "tasks": [task.to_dict() for task in self.tasks],
            "others_can_add_tasks": self.others_can_add_tasks,
            "others_can_mark_tasks_as_done": self.others_can_mark_tasks_as_done,
        }
        cl = Checklist.de_json(json_dict, offline_bot)
        assert isinstance(cl, Checklist)
        assert cl.title == self.title
        assert cl.title_entities == tuple(self.title_entities)
        assert cl.tasks == tuple(self.tasks)
        assert cl.others_can_add_tasks is self.others_can_add_tasks
        assert cl.others_can_mark_tasks_as_done is self.others_can_mark_tasks_as_done
        assert cl.api_kwargs == {}

    def test_de_json_required_fields(self, offline_bot):
        json_dict = {
            "title": self.title,
            "tasks": [task.to_dict() for task in self.tasks],
        }
        cl = Checklist.de_json(json_dict, offline_bot)
        assert isinstance(cl, Checklist)
        assert cl.title == self.title
        assert cl.title_entities == ()
        assert cl.tasks == tuple(self.tasks)
        assert not cl.others_can_add_tasks
        assert not cl.others_can_mark_tasks_as_done

    def test_parse_entity(self, checklist):
        assert checklist.parse_entity(checklist.title_entities[0]) == "Checklist"
        assert checklist.parse_entity(checklist.title_entities[1]) == "Title"

    def test_parse_entities(self, checklist):
        assert checklist.parse_entities(MessageEntity.BOLD) == {
            checklist.title_entities[0]: "Checklist"
        }
        assert checklist.parse_entities() == {
            checklist.title_entities[0]: "Checklist",
            checklist.title_entities[1]: "Title",
        }

    def test_equality(self, checklist, checklist_task):
        cl1 = checklist
        cl2 = Checklist(
            title=self.title + " other",
            tasks=[ChecklistTask(id=1, text="something"), ChecklistTask(id=2, text="something")],
        )
        cl3 = Checklist(
            title=self.title + " other",
            tasks=[ChecklistTask(id=42, text="Task 2")],
        )
        cl4 = checklist_task

        assert cl1 == cl2
        assert hash(cl1) == hash(cl2)

        assert cl1 != cl3
        assert hash(cl1) != hash(cl3)

        assert cl1 != cl4
        assert hash(cl1) != hash(cl4)


class ChecklistTasksDoneTestBase:
    checklist_message = make_message("Checklist message")
    marked_as_done_task_ids = [1, 2, 3]
    marked_as_not_done_task_ids = [4, 5]


@pytest.fixture(scope="module")
def checklist_tasks_done():
    return ChecklistTasksDone(
        checklist_message=ChecklistTasksDoneTestBase.checklist_message,
        marked_as_done_task_ids=ChecklistTasksDoneTestBase.marked_as_done_task_ids,
        marked_as_not_done_task_ids=ChecklistTasksDoneTestBase.marked_as_not_done_task_ids,
    )


class TestChecklistTasksDoneWithoutRequest(ChecklistTasksDoneTestBase):
    def test_slot_behaviour(self, checklist_tasks_done):
        for attr in checklist_tasks_done.__slots__:
            assert getattr(checklist_tasks_done, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(checklist_tasks_done)) == len(set(mro_slots(checklist_tasks_done))), (
            "duplicate slot"
        )

    def test_to_dict(self, checklist_tasks_done):
        cltd_dict = checklist_tasks_done.to_dict()
        assert isinstance(cltd_dict, dict)
        assert cltd_dict["checklist_message"] == self.checklist_message.to_dict()
        assert cltd_dict["marked_as_done_task_ids"] == self.marked_as_done_task_ids
        assert cltd_dict["marked_as_not_done_task_ids"] == self.marked_as_not_done_task_ids

    def test_de_json(self, offline_bot):
        json_dict = {
            "checklist_message": self.checklist_message.to_dict(),
            "marked_as_done_task_ids": self.marked_as_done_task_ids,
            "marked_as_not_done_task_ids": self.marked_as_not_done_task_ids,
        }
        cltd = ChecklistTasksDone.de_json(json_dict, offline_bot)
        assert isinstance(cltd, ChecklistTasksDone)
        assert cltd.checklist_message == self.checklist_message
        assert cltd.marked_as_done_task_ids == tuple(self.marked_as_done_task_ids)
        assert cltd.marked_as_not_done_task_ids == tuple(self.marked_as_not_done_task_ids)
        assert cltd.api_kwargs == {}

    def test_de_json_required_fields(self, offline_bot):
        cltd = ChecklistTasksDone.de_json({}, offline_bot)
        assert isinstance(cltd, ChecklistTasksDone)
        assert cltd.checklist_message is None
        assert cltd.marked_as_done_task_ids == ()
        assert cltd.marked_as_not_done_task_ids == ()
        assert cltd.api_kwargs == {}

    def test_equality(self, checklist_tasks_done):
        cltd1 = checklist_tasks_done
        cltd2 = ChecklistTasksDone(
            checklist_message=None,
            marked_as_done_task_ids=[1, 2, 3],
            marked_as_not_done_task_ids=[4, 5],
        )
        cltd3 = ChecklistTasksDone(
            checklist_message=make_message("Checklist message"),
            marked_as_done_task_ids=[1, 2, 3],
        )
        cltd4 = make_message("Not a checklist tasks done")

        assert cltd1 == cltd2
        assert hash(cltd1) == hash(cltd2)

        assert cltd1 != cltd3
        assert hash(cltd1) != hash(cltd3)

        assert cltd1 != cltd4
        assert hash(cltd1) != hash(cltd4)


class ChecklistTasksAddedTestBase:
    checklist_message = make_message("Checklist message")
    tasks = [
        ChecklistTask(id=1, text="Task 1"),
        ChecklistTask(id=2, text="Task 2"),
        ChecklistTask(id=3, text="Task 3"),
    ]


@pytest.fixture(scope="module")
def checklist_tasks_added():
    return ChecklistTasksAdded(
        checklist_message=ChecklistTasksAddedTestBase.checklist_message,
        tasks=ChecklistTasksAddedTestBase.tasks,
    )


class TestChecklistTasksAddedWithoutRequest(ChecklistTasksAddedTestBase):
    def test_slot_behaviour(self, checklist_tasks_added):
        for attr in checklist_tasks_added.__slots__:
            assert getattr(checklist_tasks_added, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(checklist_tasks_added)) == len(
            set(mro_slots(checklist_tasks_added))
        ), "duplicate slot"

    def test_to_dict(self, checklist_tasks_added):
        clta_dict = checklist_tasks_added.to_dict()
        assert isinstance(clta_dict, dict)
        assert clta_dict["checklist_message"] == self.checklist_message.to_dict()
        assert clta_dict["tasks"] == [task.to_dict() for task in self.tasks]

    def test_de_json(self, offline_bot):
        json_dict = {
            "checklist_message": self.checklist_message.to_dict(),
            "tasks": [task.to_dict() for task in self.tasks],
        }
        clta = ChecklistTasksAdded.de_json(json_dict, offline_bot)
        assert isinstance(clta, ChecklistTasksAdded)
        assert clta.checklist_message == self.checklist_message
        assert clta.tasks == tuple(self.tasks)
        assert clta.api_kwargs == {}

    def test_de_json_required_fields(self, offline_bot):
        clta = ChecklistTasksAdded.de_json(
            {"tasks": [task.to_dict() for task in self.tasks]}, offline_bot
        )
        assert isinstance(clta, ChecklistTasksAdded)
        assert clta.checklist_message is None
        assert clta.tasks == tuple(self.tasks)
        assert clta.api_kwargs == {}

    def test_equality(self, checklist_tasks_added):
        clta1 = checklist_tasks_added
        clta2 = ChecklistTasksAdded(
            checklist_message=None,
            tasks=[
                ChecklistTask(id=1, text="Other Task 1"),
                ChecklistTask(id=2, text="Other Task 2"),
                ChecklistTask(id=3, text="Other Task 3"),
            ],
        )
        clta3 = ChecklistTasksAdded(
            checklist_message=make_message("Checklist message"),
            tasks=[ChecklistTask(id=1, text="Task 1")],
        )
        clta4 = make_message("Not a checklist tasks added")

        assert clta1 == clta2
        assert hash(clta1) == hash(clta2)

        assert clta1 != clta3
        assert hash(clta1) != hash(clta3)

        assert clta1 != clta4
        assert hash(clta1) != hash(clta4)
